@@ -92,18 +92,24 @@
     method is raised.
 
 
-    :copyright: (c) 2011 by the Werkzeug Team, see AUTHORS for more details.
+    :copyright: (c) 2013 by the Werkzeug Team, see AUTHORS for more details.
     :license: BSD, see LICENSE for more details.
 """
 import re
 import posixpath
 from pprint import pformat
-from urlparse import urljoin
+try:
+    from urlparse import urljoin
+except ImportError:
+    from urllib.parse import urljoin
 
-from werkzeug.urls import url_encode, url_decode, url_quote
+from werkzeug.urls import url_encode, url_quote
 from werkzeug.utils import redirect, format_string
 from werkzeug.exceptions import HTTPException, NotFound, MethodNotAllowed
-from werkzeug._internal import _get_environ
+from werkzeug._internal import _get_environ, _encode_idna
+from werkzeug._compat import itervalues, iteritems, to_unicode, to_bytes, \
+     text_type, string_types, native_string_result, \
+     implements_to_string, wsgi_decoding_dance
 from werkzeug.datastructures import ImmutableDict, MultiDict
 
 
@@ -115,7 +121,7 @@ _rule_re = re.compile(r'''
         (?:\((?P<args>.*?)\))?                  # converter arguments
         \:                                      # variable delimiter
     )?
-    (?P<variable>[a-zA-Z][a-zA-Z0-9_]*)         # variable name
+    (?P<variable>[a-zA-Z_][a-zA-Z0-9_]*)        # variable name
     >
 ''', re.VERBOSE)
 _simple_rule_re = re.compile(r'<([^>]+)>')
@@ -149,7 +155,7 @@ def _pythonize(value):
             pass
     if value[:1] == value[-1:] and value[0] in '"\'':
         value = value[1:-1]
-    return unicode(value)
+    return text_type(value)
 
 
 def parse_converter_args(argstr):
@@ -203,22 +209,6 @@ def parse_rule(rule):
         yield None, None, remaining
 
 
-def get_converter(map, name, args):
-    """Create a new converter for the given arguments or raise
-    exception if the converter does not exist.
-
-    :internal:
-    """
-    if not name in map.converters:
-        raise LookupError('the converter %r does not exist' % name)
-    if args:
-        args, kwargs = parse_converter_args(args)
-    else:
-        args = ()
-        kwargs = {}
-    return map.converters[name](map, *args, **kwargs)
-
-
 class RoutingException(Exception):
     """Special exceptions that require the application to redirect, notifying
     about missing urls, etc.
@@ -240,7 +230,7 @@ class RequestRedirect(HTTPException, RoutingException):
         self.new_url = new_url
 
     def get_response(self, environ):
-        return redirect(self.new_url, 301)
+        return redirect(self.new_url, self.code)
 
 
 class RequestSlash(RoutingException):
@@ -409,14 +399,14 @@ class RuleTemplateFactory(RuleFactory):
                 new_defaults = subdomain = None
                 if rule.defaults:
                     new_defaults = {}
-                    for key, value in rule.defaults.iteritems():
-                        if isinstance(value, basestring):
+                    for key, value in iteritems(rule.defaults):
+                        if isinstance(value, string_types):
                             value = format_string(value, self.context)
                         new_defaults[key] = value
                 if rule.subdomain is not None:
                     subdomain = format_string(rule.subdomain, self.context)
                 new_endpoint = rule.endpoint
-                if isinstance(new_endpoint, basestring):
+                if isinstance(new_endpoint, string_types):
                     new_endpoint = format_string(new_endpoint, self.context)
                 yield Rule(
                     format_string(rule.rule, self.context),
@@ -429,6 +419,7 @@ class RuleTemplateFactory(RuleFactory):
                 )
 
 
+@implements_to_string
 class Rule(RuleFactory):
     """A Rule represents one URL pattern.  There are some options for `Rule`
     that change the way it behaves and are passed to the `Rule` constructor.
@@ -610,6 +601,15 @@ class Rule(RuleFactory):
             self.subdomain = map.default_subdomain
         self.compile()
 
+    def get_converter(self, variable_name, converter_name, args, kwargs):
+        """Looks up the converter for the given parameter.
+
+        .. versionadded:: 0.9
+        """
+        if not converter_name in self.map.converters:
+            raise LookupError('the converter %r does not exist' % converter_name)
+        return self.map.converters[converter_name](self.map, *args, **kwargs)
+
     def compile(self):
         """Compiles the regular expression and stores it."""
         assert self.map is not None, 'rule not bound'
@@ -633,7 +633,13 @@ class Rule(RuleFactory):
                         if part:
                             self._weights.append((0, -len(part)))
                 else:
-                    convobj = get_converter(self.map, converter, arguments)
+                    if arguments:
+                        c_args, c_kwargs = parse_converter_args(arguments)
+                    else:
+                        c_args = ()
+                        c_kwargs = {}
+                    convobj = self.get_converter(
+                        variable, converter, c_args, c_kwargs)
                     regex_parts.append('(?P<%s>%s)' % (variable, convobj.regex))
                     self._converters[variable] = convobj
                     self._trace.append((True, variable))
@@ -684,7 +690,7 @@ class Rule(RuleFactory):
                     del groups['__suffix__']
 
                 result = {}
-                for name, value in groups.iteritems():
+                for name, value in iteritems(groups):
                     try:
                         value = self._converters[name].to_python(value)
                     except ValidationError:
@@ -715,8 +721,8 @@ class Rule(RuleFactory):
                     return
                 processed.add(data)
             else:
-                add(data)
-        domain_part, url = (u''.join(tmp)).split('|', 1)
+                add(url_quote(to_bytes(data, self.map.charset), safe='/:|+'))
+        domain_part, url = (u''.join(tmp)).split(u'|', 1)
 
         if append_unknown:
             query_vars = MultiDict(values)
@@ -725,7 +731,7 @@ class Rule(RuleFactory):
                     del query_vars[key]
 
             if query_vars:
-                url += '?' + url_encode(query_vars, self.map.charset,
+                url += u'?' + url_encode(query_vars, charset=self.map.charset,
                                         sort=self.map.sort_parameters,
                                         key=self.map.sort_key)
 
@@ -762,7 +768,7 @@ class Rule(RuleFactory):
         # in case defaults are given we ensure taht either the value was
         # skipped or the value is the same as the default value.
         if defaults:
-            for key, value in defaults.iteritems():
+            for key, value in iteritems(defaults):
                 if key in values and value != values[key]:
                     return False
 
@@ -799,28 +805,24 @@ class Rule(RuleFactory):
     def __ne__(self, other):
         return not self.__eq__(other)
 
-    def __unicode__(self):
+    def __str__(self):
         return self.rule
 
-    def __str__(self):
-        charset = self.map is not None and self.map.charset or 'utf-8'
-        return unicode(self).encode(charset)
-
+    @native_string_result
     def __repr__(self):
         if self.map is None:
-            return '<%s (unbound)>' % self.__class__.__name__
-        charset = self.map is not None and self.map.charset or 'utf-8'
+            return u'<%s (unbound)>' % self.__class__.__name__
         tmp = []
         for is_dynamic, data in self._trace:
             if is_dynamic:
-                tmp.append('<%s>' % data)
+                tmp.append(u'<%s>' % data)
             else:
                 tmp.append(data)
-        return '<%s %r%s -> %s>' % (
+        return u'<%s %s%s -> %s>' % (
             self.__class__.__name__,
-            (u''.join(tmp).encode(charset)).lstrip('|'),
-            self.methods is not None and ' (%s)' % \
-                ', '.join(self.methods) or '',
+            repr((u''.join(tmp)).lstrip(u'|')).lstrip(u'u'),
+            self.methods is not None and u' (%s)' % \
+                u', '.join(self.methods) or u'',
             self.endpoint
         )
 
@@ -837,7 +839,7 @@ class BaseConverter(object):
         return value
 
     def to_url(self, value):
-        return url_quote(value, self.map.charset)
+        return url_quote(to_bytes(value, self.map.charset))
 
 
 class UnicodeConverter(BaseConverter):
@@ -1108,7 +1110,11 @@ class Map(object):
 
         .. versionadded:: 0.7
            `query_args` added
+
+        .. versionadded:: 0.8
+           `query_args` can now also be a string.
         """
+        server_name = server_name.lower()
         if self.host_matching:
             if subdomain is not None:
                 raise RuntimeError('host matching enabled and a '
@@ -1117,6 +1123,7 @@ class Map(object):
             subdomain = self.default_subdomain
         if script_name is None:
             script_name = '/'
+        server_name = _encode_idna(server_name)
         return MapAdapter(self, server_name, script_name, subdomain,
                           url_scheme, path_info, default_method, query_args)
 
@@ -1146,6 +1153,10 @@ class Map(object):
             parameter that did not have any effect.  It was removed because
             of that.
 
+        .. versionchanged:: 0.8
+           This will no longer raise a ValueError when an unexpected server
+           name was passed.
+
         :param environ: a WSGI environment.
         :param server_name: an optional server name hint (see above).
         :param subdomain: optionally the current subdomain (see above).
@@ -1160,6 +1171,7 @@ class Map(object):
                    in (('https', '443'), ('http', '80')):
                     server_name += ':' + environ['SERVER_PORT']
         elif subdomain is None and not self.host_matching:
+            server_name = server_name.lower()
             if 'HTTP_HOST' in environ:
                 wsgi_server_name = environ.get('HTTP_HOST')
             else:
@@ -1167,21 +1179,31 @@ class Map(object):
                 if (environ['wsgi.url_scheme'], environ['SERVER_PORT']) not \
                    in (('https', '443'), ('http', '80')):
                     wsgi_server_name += ':' + environ['SERVER_PORT']
+            wsgi_server_name = wsgi_server_name.lower()
             cur_server_name = wsgi_server_name.split('.')
             real_server_name = server_name.split('.')
             offset = -len(real_server_name)
             if cur_server_name[offset:] != real_server_name:
-                raise ValueError('the server name provided (%r) does not '
-                                 'match the server name from the WSGI '
-                                 'environment (%r)' %
-                                 (server_name, wsgi_server_name))
-            subdomain = '.'.join(filter(None, cur_server_name[:offset]))
-        query_args = url_decode(environ.get('QUERY_STRING', ''),
-                                charset=self.charset,
-                                errors=self.encoding_errors)
-        return Map.bind(self, server_name, environ.get('SCRIPT_NAME'),
+                # This can happen even with valid configs if the server was
+                # accesssed directly by IP address under some situations.
+                # Instead of raising an exception like in Werkzeug 0.7 or
+                # earlier we go by an invalid subdomain which will result
+                # in a 404 error on matching.
+                subdomain = '<invalid>'
+            else:
+                subdomain = '.'.join(filter(None, cur_server_name[:offset]))
+
+        def _get_wsgi_string(name):
+            val = environ.get(name)
+            if val is not None:
+                return wsgi_decoding_dance(val, self.charset)
+
+        script_name = _get_wsgi_string('SCRIPT_NAME')
+        path_info = _get_wsgi_string('PATH_INFO')
+        query_args = _get_wsgi_string('QUERY_STRING')
+        return Map.bind(self, server_name, script_name,
                         subdomain, environ['wsgi.url_scheme'],
-                        environ['REQUEST_METHOD'], environ.get('PATH_INFO'),
+                        environ['REQUEST_METHOD'], path_info,
                         query_args=query_args)
 
     def update(self):
@@ -1190,7 +1212,7 @@ class Map(object):
         """
         if self._remap:
             self._rules.sort(key=lambda x: x.match_compare_key())
-            for rules in self._rules_by_endpoint.itervalues():
+            for rules in itervalues(self._rules_by_endpoint):
                 rules.sort(key=lambda x: x.build_compare_key())
             self._remap = False
 
@@ -1207,14 +1229,15 @@ class MapAdapter(object):
     def __init__(self, map, server_name, script_name, subdomain,
                  url_scheme, path_info, default_method, query_args=None):
         self.map = map
-        self.server_name = server_name
-        if not script_name.endswith('/'):
-            script_name += '/'
+        self.server_name = to_unicode(server_name)
+        script_name = to_unicode(script_name)
+        if not script_name.endswith(u'/'):
+            script_name += u'/'
         self.script_name = script_name
-        self.subdomain = subdomain
-        self.url_scheme = url_scheme
-        self.path_info = path_info or u''
-        self.default_method = default_method
+        self.subdomain = to_unicode(subdomain)
+        self.url_scheme = to_unicode(url_scheme)
+        self.path_info = to_unicode(path_info)
+        self.default_method = to_unicode(default_method)
         self.query_args = query_args
 
     def dispatch(self, view_func, path_info=None, method=None,
@@ -1264,10 +1287,10 @@ class MapAdapter(object):
         try:
             try:
                 endpoint, args = self.match(path_info, method)
-            except RequestRedirect, e:
+            except RequestRedirect as e:
                 return e
             return view_func(endpoint, args)
-        except HTTPException, e:
+        except HTTPException as e:
             if catch_http_exceptions:
                 return e
             raise
@@ -1337,21 +1360,24 @@ class MapAdapter(object):
         :param return_rule: return the rule that matched instead of just the
                             endpoint (defaults to `False`).
         :param query_args: optional query arguments that are used for
-                           automatic redirects.  It's currently not possible
-                           to use the query arguments for URL matching.
+                           automatic redirects as string or dictionary.  It's
+                           currently not possible to use the query arguments
+                           for URL matching.
 
         .. versionadded:: 0.6
            `return_rule` was added.
 
         .. versionadded:: 0.7
            `query_args` was added.
+
+        .. versionchanged:: 0.8
+           `query_args` can now also be a string.
         """
         self.map.update()
         if path_info is None:
             path_info = self.path_info
-        if not isinstance(path_info, unicode):
-            path_info = path_info.decode(self.map.charset,
-                                         self.map.encoding_errors)
+        else:
+            path_info = to_unicode(path_info, self.map.charset)
         if query_args is None:
             query_args = self.query_args
         method = (method or self.default_method).upper()
@@ -1366,7 +1392,7 @@ class MapAdapter(object):
             except RequestSlash:
                 raise RequestRedirect(self.make_redirect_url(
                     path_info + '/', query_args))
-            except RequestAliasRedirect, e:
+            except RequestAliasRedirect as e:
                 raise RequestRedirect(self.make_alias_redirect_url(
                     path, rule.endpoint, e.matched_values, method, query_args))
             if rv is None:
@@ -1382,7 +1408,7 @@ class MapAdapter(object):
                     raise RequestRedirect(redirect_url)
 
             if rule.redirect_to is not None:
-                if isinstance(rule.redirect_to, basestring):
+                if isinstance(rule.redirect_to, string_types):
                     def _handle_match(match):
                         value = rv[match.group(1)]
                         return rule._converters[match.group(1)].to_url(value)
@@ -1430,9 +1456,9 @@ class MapAdapter(object):
         """
         try:
             self.match(path_info, method='--')
-        except MethodNotAllowed, e:
+        except MethodNotAllowed as e:
             return e.valid_methods
-        except HTTPException, e:
+        except HTTPException as e:
             pass
         return []
 
@@ -1444,11 +1470,13 @@ class MapAdapter(object):
         if self.map.host_matching:
             if domain_part is None:
                 return self.server_name
-            return domain_part
+            return to_unicode(domain_part, 'ascii')
         subdomain = domain_part
         if subdomain is None:
             subdomain = self.subdomain
-        return (subdomain and subdomain + '.' or '') + self.server_name
+        else:
+            subdomain = to_unicode(subdomain, 'ascii')
+        return (subdomain and subdomain + u'.' or u'') + self.server_name
 
     def get_default_redirect(self, rule, method, values, query_args):
         """A helper that returns the URL to redirect to if it finds one.
@@ -1470,6 +1498,11 @@ class MapAdapter(object):
                 return self.make_redirect_url(
                     path, query_args, domain_part=domain_part)
 
+    def encode_query_args(self, query_args):
+        if not isinstance(query_args, string_types):
+            query_args = url_encode(query_args, self.map.charset)
+        return query_args
+
     def make_redirect_url(self, path_info, query_args=None, domain_part=None):
         """Creates a redirect URL.
 
@@ -1477,12 +1510,13 @@ class MapAdapter(object):
         """
         suffix = ''
         if query_args:
-            suffix = '?' + url_encode(query_args, self.map.charset)
+            suffix = '?' + self.encode_query_args(query_args)
         return str('%s://%s/%s%s' % (
             self.url_scheme,
             self.get_host(domain_part),
             posixpath.join(self.script_name[:-1].lstrip('/'),
-                           url_quote(path_info.lstrip('/'), self.map.charset)),
+                           url_quote(path_info.lstrip('/'), self.map.charset,
+                                     safe='/:|+')),
             suffix
         ))
 
@@ -1491,7 +1525,7 @@ class MapAdapter(object):
         url = self.build(endpoint, values, method, append_unknown=False,
                          force_external=True)
         if query_args:
-            url += '?' + url_encode(query_args, self.map.charset)
+            url += '?' + self.encode_query_args(query_args)
         assert url != path, 'detected invalid alias setting.  No canonical ' \
                'URL found'
         return url
@@ -1576,7 +1610,7 @@ class MapAdapter(object):
             if isinstance(values, MultiDict):
                 valueiter = values.iteritems(multi=True)
             else:
-                valueiter = values.iteritems()
+                valueiter = iteritems(values)
             values = dict((k, v) for k, v in valueiter if v is not None)
         else:
             values = {}
