@@ -5,20 +5,26 @@
 
     This module implements WSGI related helpers.
 
-    :copyright: (c) 2011 by the Werkzeug Team, see AUTHORS for more details.
+    :copyright: (c) 2013 by the Werkzeug Team, see AUTHORS for more details.
     :license: BSD, see LICENSE for more details.
 """
+import re
 import os
-import urllib
-import urlparse
+import sys
 import posixpath
 import mimetypes
+from itertools import chain
 from zlib import adler32
 from time import time, mktime
 from datetime import datetime
+from functools import partial, update_wrapper
 
-from werkzeug._internal import _patch_wrapper
+from werkzeug._compat import iteritems, text_type, string_types, \
+     implements_iterator, make_literal_wrapper, to_unicode, to_bytes, \
+     wsgi_get_bytes, try_coerce_native
+from werkzeug._internal import _empty_stream, _encode_idna
 from werkzeug.http import is_resource_modified, http_date
+from werkzeug.urls import uri_to_iri, url_quote, url_parse, url_join
 
 
 def responder(f):
@@ -31,11 +37,11 @@ def responder(f):
         def application(environ, start_response):
             return Response('Hello World!')
     """
-    return _patch_wrapper(f, lambda *a: f(*a)(*a[-2:]))
+    return update_wrapper(lambda *a: f(*a)(*a[-2:]), f)
 
 
 def get_current_url(environ, root_only=False, strip_querystring=False,
-                    host_only=False):
+                    host_only=False, trusted_hosts=None):
     """A handy helper function that recreates the full URL for the current
     request or parts of it.  Here an example:
 
@@ -50,48 +56,202 @@ def get_current_url(environ, root_only=False, strip_querystring=False,
     >>> get_current_url(env, strip_querystring=True)
     'http://localhost/script/'
 
+    This optionally it verifies that the host is in a list of trusted hosts.
+    If the host is not in there it will raise a
+    :exc:`~werkzeug.exceptions.SecurityError`.
+
     :param environ: the WSGI environment to get the current URL from.
     :param root_only: set `True` if you only want the root URL.
     :param strip_querystring: set to `True` if you don't want the querystring.
     :param host_only: set to `True` if the host URL should be returned.
+    :param trusted_hosts: a list of trusted hosts, see :func:`host_is_trusted`
+                          for more information.
     """
-    tmp = [environ['wsgi.url_scheme'], '://', get_host(environ)]
+    tmp = [environ['wsgi.url_scheme'], '://', get_host(environ, trusted_hosts)]
     cat = tmp.append
     if host_only:
-        return ''.join(tmp) + '/'
-    cat(urllib.quote(environ.get('SCRIPT_NAME', '').rstrip('/')))
-    if root_only:
-        cat('/')
-    else:
-        cat(urllib.quote('/' + environ.get('PATH_INFO', '').lstrip('/')))
+        return uri_to_iri(''.join(tmp) + '/')
+    cat(url_quote(wsgi_get_bytes(environ.get('SCRIPT_NAME', ''))).rstrip('/'))
+    cat('/')
+    if not root_only:
+        cat(url_quote(wsgi_get_bytes(environ.get('PATH_INFO', '')).lstrip(b'/')))
         if not strip_querystring:
-            qs = environ.get('QUERY_STRING')
+            qs = get_query_string(environ)
             if qs:
                 cat('?' + qs)
-    return ''.join(tmp)
+    return uri_to_iri(''.join(tmp))
 
 
-def get_host(environ):
+def host_is_trusted(hostname, trusted_list):
+    """Checks if a host is trusted against a list.  This also takes care
+    of port normalization.
+
+    .. versionadded:: 0.9
+
+    :param hostname: the hostname to check
+    :param trusted_list: a list of hostnames to check against.  If a
+                         hostname starts with a dot it will match against
+                         all subdomains as well.
+    """
+    if not hostname:
+        return False
+
+    if isinstance(trusted_list, string_types):
+        trusted_list = [trusted_list]
+
+    def _normalize(hostname):
+        if ':' in hostname:
+            hostname = hostname.rsplit(':', 1)[0]
+        return _encode_idna(hostname)
+
+    hostname = _normalize(hostname)
+    for ref in trusted_list:
+        if ref.startswith('.'):
+            ref = ref[1:]
+            suffix_match = True
+        else:
+            suffix_match = False
+        ref = _normalize(ref)
+        if ref == hostname:
+            return True
+        if suffix_match and hostname.endswith('.' + ref):
+            return True
+    return False
+
+
+def get_host(environ, trusted_hosts=None):
     """Return the real host for the given WSGI environment.  This takes care
-    of the `X-Forwarded-Host` header.
+    of the `X-Forwarded-Host` header.  Optionally it verifies that the host
+    is in a list of trusted hosts.  If the host is not in there it will raise
+    a :exc:`~werkzeug.exceptions.SecurityError`.
 
     :param environ: the WSGI environment to get the host of.
+    :param trusted_hosts: a list of trusted hosts, see :func:`host_is_trusted`
+                          for more information.
     """
     if 'HTTP_X_FORWARDED_HOST' in environ:
-        return environ['HTTP_X_FORWARDED_HOST']
+        rv = environ['HTTP_X_FORWARDED_HOST'].split(',')[0].strip()
     elif 'HTTP_HOST' in environ:
-        return environ['HTTP_HOST']
-    result = environ['SERVER_NAME']
-    if (environ['wsgi.url_scheme'], environ['SERVER_PORT']) not \
-       in (('https', '443'), ('http', '80')):
-        result += ':' + environ['SERVER_PORT']
-    return result
+        rv = environ['HTTP_HOST']
+    else:
+        rv = environ['SERVER_NAME']
+        if (environ['wsgi.url_scheme'], environ['SERVER_PORT']) not \
+           in (('https', '443'), ('http', '80')):
+            rv += ':' + environ['SERVER_PORT']
+    if trusted_hosts is not None:
+        if not host_is_trusted(rv, trusted_hosts):
+            from werkzeug.exceptions import SecurityError
+            raise SecurityError('Host "%s" is not trusted' % rv)
+    return rv
 
 
-def pop_path_info(environ):
+def get_content_length(environ):
+    """Returns the content length from the WSGI environment as
+    integer.  If it's not available `None` is returned.
+
+    .. versionadded:: 0.9
+
+    :param environ: the WSGI environ to fetch the content length from.
+    """
+    content_length = environ.get('CONTENT_LENGTH')
+    if content_length is not None:
+        try:
+            return max(0, int(content_length))
+        except (ValueError, TypeError):
+            pass
+
+
+def get_input_stream(environ, safe_fallback=True):
+    """Returns the input stream from the WSGI environment and wraps it
+    in the most sensible way possible.  The stream returned is not the
+    raw WSGI stream in most cases but one that is safe to read from
+    without taking into account the content length.
+
+    .. versionadded:: 0.9
+
+    :param environ: the WSGI environ to fetch the stream from.
+    :param safe: indicates weather the function should use an empty
+                 stream as safe fallback or just return the original
+                 WSGI input stream if it can't wrap it safely.  The
+                 default is to return an empty string in those cases.
+    """
+    stream = environ['wsgi.input']
+    content_length = get_content_length(environ)
+
+    # A wsgi extension that tells us if the input is terminated.  In
+    # that case we return the stream unchanged as we know we can savely
+    # read it until the end.
+    if environ.get('wsgi.input_terminated'):
+        return stream
+
+    # If we don't have a content length we fall back to an empty stream
+    # in case of a safe fallback, otherwise we return the stream unchanged.
+    # The non-safe fallback is not recommended but might be useful in
+    # some situations.
+    if content_length is None:
+        return safe_fallback and _empty_stream or stream
+
+    # Otherwise limit the stream to the content length
+    return LimitedStream(stream, content_length)
+
+
+def get_query_string(environ):
+    """Returns the `QUERY_STRING` from the WSGI environment.  This also takes
+    care about the WSGI decoding dance on Python 3 environments as a
+    native string.  The string returned will be restricted to ASCII
+    characters.
+
+    .. versionadded:: 0.9
+
+    :param environ: the WSGI environment object to get the query string from.
+    """
+    qs = wsgi_get_bytes(environ.get('QUERY_STRING', ''))
+    # QUERY_STRING really should be ascii safe but some browsers
+    # will send us some unicode stuff (I am looking at you IE).
+    # In that case we want to urllib quote it badly.
+    return try_coerce_native(url_quote(qs, safe=':&%=+$!*\'(),'))
+
+
+def get_path_info(environ, charset='utf-8', errors='replace'):
+    """Returns the `PATH_INFO` from the WSGI environment and properly
+    decodes it.  This also takes care about the WSGI decoding dance
+    on Python 3 environments.  if the `charset` is set to `None` a
+    bytestring is returned.
+
+    .. versionadded:: 0.9
+
+    :param environ: the WSGI environment object to get the path from.
+    :param charset: the charset for the path info, or `None` if no
+                    decoding should be performed.
+    :param errors: the decoding error handling.
+    """
+    path = wsgi_get_bytes(environ.get('PATH_INFO', ''))
+    return to_unicode(path, charset, errors, allow_none_charset=True)
+
+
+def get_script_name(environ, charset='utf-8', errors='replace'):
+    """Returns the `SCRIPT_NAME` from the WSGI environment and properly
+    decodes it.  This also takes care about the WSGI decoding dance
+    on Python 3 environments.  if the `charset` is set to `None` a
+    bytestring is returned.
+
+    .. versionadded:: 0.9
+
+    :param environ: the WSGI environment object to get the path from.
+    :param charset: the charset for the path, or `None` if no
+                    decoding should be performed.
+    :param errors: the decoding error handling.
+    """
+    path = wsgi_get_bytes(environ.get('SCRIPT_NAME', ''))
+    return to_unicode(path, charset, errors, allow_none_charset=True)
+
+
+def pop_path_info(environ, charset='utf-8', errors='replace'):
     """Removes and returns the next segment of `PATH_INFO`, pushing it onto
     `SCRIPT_NAME`.  Returns `None` if there is nothing left on `PATH_INFO`.
 
+    If the `charset` is set to `None` a bytestring is returned.
+
     If there are empty segments (``'/foo//bar``) these are ignored but
     properly pushed to the `SCRIPT_NAME`:
 
@@ -107,6 +267,10 @@ def pop_path_info(environ):
 
     .. versionadded:: 0.5
 
+    .. versionchanged:: 0.9
+       The path is now decoded and a charset and encoding
+       parameter can be provided.
+
     :param environ: the WSGI environment that is modified.
     """
     path = environ.get('PATH_INFO')
@@ -124,15 +288,17 @@ def pop_path_info(environ):
     if '/' not in path:
         environ['PATH_INFO'] = ''
         environ['SCRIPT_NAME'] = script_name + path
-        return path
+        rv = wsgi_get_bytes(path)
+    else:
+        segment, path = path.split('/', 1)
+        environ['PATH_INFO'] = '/' + path
+        environ['SCRIPT_NAME'] = script_name + segment
+        rv = wsgi_get_bytes(segment)
 
-    segment, path = path.split('/', 1)
-    environ['PATH_INFO'] = '/' + path
-    environ['SCRIPT_NAME'] = script_name + segment
-    return segment
+    return to_unicode(rv, charset, errors, allow_none_charset=True)
 
 
-def peek_path_info(environ):
+def peek_path_info(environ, charset='utf-8', errors='replace'):
     """Returns the next segment on the `PATH_INFO` or `None` if there
     is none.  Works like :func:`pop_path_info` without modifying the
     environment:
@@ -143,13 +309,20 @@ def peek_path_info(environ):
     >>> peek_path_info(env)
     'a'
 
+    If the `charset` is set to `None` a bytestring is returned.
+
     .. versionadded:: 0.5
 
+    .. versionchanged:: 0.9
+       The path is now decoded and a charset and encoding
+       parameter can be provided.
+
     :param environ: the WSGI environment that is checked.
     """
     segments = environ.get('PATH_INFO', '').lstrip('/').split('/', 1)
     if segments:
-        return segments[0]
+        return to_unicode(wsgi_get_bytes(segments[0]),
+                          charset, errors, allow_none_charset=True)
 
 
 def extract_path_info(environ_or_baseurl, path_or_url, charset='utf-8',
@@ -190,13 +363,6 @@ def extract_path_info(environ_or_baseurl, path_or_url, charset='utf-8',
                                   same server point to the same
                                   resource.
     """
-    from werkzeug.urls import uri_to_iri, url_fix
-
-    def _as_iri(obj):
-        if not isinstance(obj, unicode):
-            return uri_to_iri(obj, charset, errors)
-        return obj
-
     def _normalize_netloc(scheme, netloc):
         parts = netloc.split(u'@', 1)[-1].split(u':', 1)
         if len(parts) == 2:
@@ -212,15 +378,14 @@ def extract_path_info(environ_or_baseurl, path_or_url, charset='utf-8',
         return netloc
 
     # make sure whatever we are working on is a IRI and parse it
-    path = _as_iri(path_or_url)
+    path = uri_to_iri(path_or_url, charset, errors)
     if isinstance(environ_or_baseurl, dict):
         environ_or_baseurl = get_current_url(environ_or_baseurl,
                                              root_only=True)
-    base_iri = _as_iri(environ_or_baseurl)
-    base_scheme, base_netloc, base_path, = \
-        urlparse.urlsplit(base_iri)[:3]
+    base_iri = uri_to_iri(environ_or_baseurl, charset, errors)
+    base_scheme, base_netloc, base_path = url_parse(base_iri)[:3]
     cur_scheme, cur_netloc, cur_path, = \
-        urlparse.urlsplit(urlparse.urljoin(base_iri, path))[:3]
+        url_parse(url_join(base_iri, path))[:3]
 
     # normalize the network location
     base_netloc = _normalize_netloc(base_scheme, base_netloc)
@@ -310,10 +475,10 @@ class SharedDataMiddleware(object):
         self.exports = {}
         self.cache = cache
         self.cache_timeout = cache_timeout
-        for key, value in exports.iteritems():
+        for key, value in iteritems(exports):
             if isinstance(value, tuple):
                 loader = self.get_package_loader(*value)
-            elif isinstance(value, basestring):
+            elif isinstance(value, string_types):
                 if os.path.isfile(value):
                     loader = self.get_file_loader(value)
                 else:
@@ -382,7 +547,7 @@ class SharedDataMiddleware(object):
         return 'wzsdm-%d-%s-%s' % (
             mktime(mtime.timetuple()),
             file_size,
-            adler32(real_filename) & 0xffffffff
+            adler32(real_filename.encode(sys.getfilesystemencoding())) & 0xffffffff
         )
 
     def __call__(self, environ, start_response):
@@ -394,7 +559,7 @@ class SharedDataMiddleware(object):
         path = '/'.join([''] + [x for x in cleaned_path.split('/')
                                 if x and x != '..'])
         file_loader = None
-        for search_path, loader in self.exports.iteritems():
+        for search_path, loader in iteritems(self.exports):
             if search_path == path:
                 real_filename, file_loader = loader(None)
                 if file_loader is not None:
@@ -469,6 +634,7 @@ class DispatcherMiddleware(object):
         return app(environ, start_response)
 
 
+@implements_iterator
 class ClosingIterator(object):
     """The WSGI specification requires that all middlewares and gateways
     respect the `close` callback of an iterator.  Because it is useful to add
@@ -492,7 +658,7 @@ class ClosingIterator(object):
 
     def __init__(self, iterable, callbacks=None):
         iterator = iter(iterable)
-        self._next = iterator.next
+        self._next = partial(next, iterator)
         if callbacks is None:
             callbacks = []
         elif callable(callbacks):
@@ -507,7 +673,7 @@ class ClosingIterator(object):
     def __iter__(self):
         return self
 
-    def next(self):
+    def __next__(self):
         return self._next()
 
     def close(self):
@@ -534,6 +700,7 @@ def wrap_file(environ, file, buffer_size=8192):
     return environ.get('wsgi.file_wrapper', FileWrapper)(file, buffer_size)
 
 
+@implements_iterator
 class FileWrapper(object):
     """This class can be used to convert a :class:`file`-like object into
     an iterable.  It yields `buffer_size` blocks until the file is fully
@@ -563,13 +730,33 @@ class FileWrapper(object):
     def __iter__(self):
         return self
 
-    def next(self):
+    def __next__(self):
         data = self.file.read(self.buffer_size)
         if data:
             return data
         raise StopIteration()
 
 
+def _make_chunk_iter(stream, limit, buffer_size):
+    """Helper for the line and chunk iter functions."""
+    if isinstance(stream, (bytes, bytearray, text_type)):
+        raise TypeError('Passed a string or byte object instead of '
+                        'true iterator or stream.')
+    if not hasattr(stream, 'read'):
+        for item in stream:
+            if item:
+                yield item
+        return
+    if not isinstance(stream, LimitedStream) and limit is not None:
+        stream = LimitedStream(stream, limit)
+    _read = stream.read
+    while 1:
+        item = _read(buffer_size)
+        if not item:
+            break
+        yield item
+
+
 def make_line_iter(stream, limit=None, buffer_size=10 * 1024):
     """Safely iterates line-based over an input stream.  If the input stream
     is not a :class:`LimitedStream` the `limit` parameter is mandatory.
@@ -583,42 +770,116 @@ def make_line_iter(stream, limit=None, buffer_size=10 * 1024):
     If you need line-by-line processing it's strongly recommended to iterate
     over the input stream using this helper function.
 
-    :param stream: the stream to iterate over.
+    .. versionchanged:: 0.8
+       This function now ensures that the limit was reached.
+
+    .. versionadded:: 0.9
+       added support for iterators as input stream.
+
+    :param stream: the stream or iterate to iterate over.
     :param limit: the limit in bytes for the stream.  (Usually
                   content length.  Not necessary if the `stream`
                   is a :class:`LimitedStream`.
     :param buffer_size: The optional buffer size.
     """
-    if not isinstance(stream, LimitedStream):
-        if limit is None:
-            raise TypeError('stream not limited and no limit provided.')
-        stream = LimitedStream(stream, limit)
-    _read = stream.read
+    _iter = _make_chunk_iter(stream, limit, buffer_size)
+
+    first_item = next(_iter, '')
+    if not first_item:
+        return
+
+    s = make_literal_wrapper(first_item)
+    empty = s('')
+    cr = s('\r')
+    lf = s('\n')
+    crlf = s('\r\n')
+
+    _iter = chain((first_item,), _iter)
+
+    def _iter_basic_lines():
+        _join = empty.join
+        buffer = []
+        while 1:
+            new_data = next(_iter, '')
+            if not new_data:
+                break
+            new_buf = []
+            for item in chain(buffer, new_data.splitlines(True)):
+                new_buf.append(item)
+                if item and item[-1:] in crlf:
+                    yield _join(new_buf)
+                    new_buf = []
+            buffer = new_buf
+        if buffer:
+            yield _join(buffer)
+
+    # This hackery is necessary to merge 'foo\r' and '\n' into one item
+    # of 'foo\r\n' if we were unlucky and we hit a chunk boundary.
+    previous = empty
+    for item in _iter_basic_lines():
+        if item == lf and previous[-1:] == cr:
+            previous += item
+            item = empty
+        if previous:
+            yield previous
+        previous = item
+    if previous:
+        yield previous
+
+
+def make_chunk_iter(stream, separator, limit=None, buffer_size=10 * 1024):
+    """Works like :func:`make_line_iter` but accepts a separator
+    which divides chunks.  If you want newline based processing
+    you should use :func:`make_line_iter` instead as it
+    supports arbitrary newline markers.
+
+    .. versionadded:: 0.8
+
+    .. versionadded:: 0.9
+       added support for iterators as input stream.
+
+    :param stream: the stream or iterate to iterate over.
+    :param separator: the separator that divides chunks.
+    :param limit: the limit in bytes for the stream.  (Usually
+                  content length.  Not necessary if the `stream`
+                  is otherwise already limited).
+    :param buffer_size: The optional buffer size.
+    """
+    _iter = _make_chunk_iter(stream, limit, buffer_size)
+
+    first_item = next(_iter, '')
+    if not first_item:
+        return
+
+    _iter = chain((first_item,), _iter)
+    if isinstance(first_item, text_type):
+        separator = to_unicode(separator)
+        _split = re.compile(r'(%s)' % re.escape(separator)).split
+        _join = u''.join
+    else:
+        separator = to_bytes(separator)
+        _split = re.compile(b'(' + re.escape(separator) + b')').split
+        _join = b''.join
+
     buffer = []
     while 1:
-        if len(buffer) > 1:
-            yield buffer.pop()
-            continue
-
-        # we reverse the chunks because popping from the last
-        # position of the list is O(1) and the number of chunks
-        # read will be quite large for binary files.
-        chunks = _read(buffer_size).splitlines(True)
-        chunks.reverse()
-
-        first_chunk = buffer and buffer[0] or ''
-        if chunks:
-            if first_chunk.endswith('\n') or first_chunk.endswith('\r'):
-                yield first_chunk
-                first_chunk = ''
-            first_chunk += chunks.pop()
-        if not first_chunk:
-            return
-
-        buffer = chunks
-        yield first_chunk
+        new_data = next(_iter, '')
+        if not new_data:
+            break
+        chunks = _split(new_data)
+        new_buf = []
+        for item in chain(buffer, chunks):
+            if item == separator:
+                yield _join(new_buf)
+                new_buf = []
+            else:
+                new_buf.append(item)
+        buffer = new_buf
+    if buffer:
+        yield _join(buffer)
 
 
+@implements_iterator
 class LimitedStream(object):
     """Wraps a stream so that it doesn't read more than n bytes.  If the
     stream is exhausted and the caller tries to get more bytes from it
@@ -631,14 +892,6 @@ class LimitedStream(object):
     output.  Otherwise :meth:`readlines` will try to read past the
     limit.
 
-    The `silent` parameter has no effect if :meth:`is_exhausted` is
-    overriden by a subclass.
-
-    .. versionchanged:: 0.6
-       Non-silent usage was deprecated because it causes confusion.
-       If you want that, override :meth:`is_exhausted` and raise a
-       :exc:`~exceptions.BadRequest` yourself.
-
     .. admonition:: Note on WSGI compliance
 
        calls to :meth:`readline` and :meth:`readlines` are not
@@ -659,22 +912,13 @@ class LimitedStream(object):
     :param limit: the limit for the stream, must not be longer than
                   what the string can provide if the stream does not
                   end with `EOF` (like `wsgi.input`)
-    :param silent: If set to `True` the stream will allow reading
-                   past the limit and will return an empty string.
     """
 
-    def __init__(self, stream, limit, silent=True):
+    def __init__(self, stream, limit):
         self._read = stream.read
         self._readline = stream.readline
         self._pos = 0
         self.limit = limit
-        self.silent = silent
-        if not silent:
-            from warnings import warn
-            warn(DeprecationWarning('non-silent usage of the '
-            'LimitedStream is deprecated.  If you want to '
-            'continue to use the stream in non-silent usage '
-            'override on_exhausted.'), stacklevel=2)
 
     def __iter__(self):
         return self
@@ -688,13 +932,19 @@ class LimitedStream(object):
         """This is called when the stream tries to read past the limit.
         The return value of this function is returned from the reading
         function.
-
-        Per default this raises a :exc:`~werkzeug.exceptions.BadRequest`.
         """
-        if self.silent:
-            return ''
-        from werkzeug.exceptions import BadRequest
-        raise BadRequest('input stream exhausted')
+        # Read null bytes from the stream so that we get the
+        # correct end of stream marker.
+        return self._read(0)
+
+    def on_disconnect(self):
+        """What should happen if a disconnect is detected?  The return
+        value of this function is returned from read functions in case
+        the client went away.  By default a
+        :exc:`~werkzeug.exceptions.ClientDisconnected` exception is raised.
+        """
+        from werkzeug.exceptions import ClientDisconnected
+        raise ClientDisconnected()
 
     def exhaust(self, chunk_size=1024 * 16):
         """Exhaust the stream.  This consumes all the data left until the
@@ -718,9 +968,15 @@ class LimitedStream(object):
         """
         if self._pos >= self.limit:
             return self.on_exhausted()
-        if size is None:
+        if size is None or size == -1:  # -1 is for consistence with file
             size = self.limit
-        read = self._read(min(self.limit - self._pos, size))
+        to_read = min(self.limit - self._pos, size)
+        try:
+            read = self._read(to_read)
+        except (IOError, ValueError):
+            return self.on_disconnect()
+        if to_read and len(read) != to_read:
+            return self.on_disconnect()
         self._pos += len(read)
         return read
 
@@ -732,7 +988,12 @@ class LimitedStream(object):
             size = self.limit - self._pos
         else:
             size = min(size, self.limit - self._pos)
-        line = self._readline(size)
+        try:
+            line = self._readline(size)
+        except (ValueError, IOError):
+            return self.on_disconnect()
+        if size and not line:
+            return self.on_disconnect()
         self._pos += len(line)
         return line
 
@@ -758,8 +1019,15 @@ class LimitedStream(object):
                 last_pos = self._pos
         return result
 
-    def next(self):
+    def tell(self):
+        """Returns the position of the stream.
+
+        .. versionadded:: 0.9
+        """
+        return self._pos
+
+    def __next__(self):
         line = self.readline()
-        if line is None:
+        if not line:
             raise StopIteration()
         return line
